package idi.gate.associationrules;

import idi.gate.associationrules.rule.Rule;

import java.util.ArrayList;

import javax.swing.BoxLayout;
import javax.swing.JScrollPane;
import javax.swing.SwingUtilities;
import javax.swing.table.AbstractTableModel;

import gate.*;
import gate.creole.AbstractVisualResource;
import gate.event.ProgressListener;
import gate.swing.XJTable;

@SuppressWarnings("serial")
public class AssociationRulesViewer extends AbstractVisualResource implements ProgressListener {
	
	private AssociationRulesWrapper target;
	private ArrayList<Rule> associationRules;
	private XJTable arTable;
	private ArTableModel arTableModel;
	
	public Resource init(){
		associationRules = new ArrayList<Rule>();
		initGuiComponents();
		return this;
	}
	
	protected void initGuiComponents(){
		setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
		arTableModel = new ArTableModel();
		arTable = new XJTable(arTableModel);
		add(new JScrollPane(arTable));
	}
	
	public void setTarget(Object target) {
	    if(!(target instanceof AssociationRulesWrapper)){
	        throw new IllegalArgumentException(
	          "The association rules viewer can only be used with an AssociationRulesGenerator!\n" +
	          target.getClass().toString() + " is not an AssociationRulesGenerator!");
	      }
	      this.target = (AssociationRulesWrapper)target;
	      this.target.addProgressListener(this);		
	}
	
	public void processFinished() {
		updateDisplay();
	}
	
	protected void updateDisplay() {
		associationRules.clear();
		if (target != null) {
			ArrayList<Rule> list = target.getAssociationRules();
			associationRules.addAll(list);
			SwingUtilities.invokeLater(new Runnable(){
				public void run(){
					arTableModel.fireTableDataChanged();
				}
			});
		}
	}

	public void progressChanged(int arg0) {
		// TODO Auto-generated method stub
		
	}
	
	protected class ArTableModel extends AbstractTableModel{
		

		static private final int DOC_RULE_COLUMN = 0;
		static private final int DOC_CONFIDENCE_COLUMN = 1;
		static private final int DOC_SUPPORT_COLUMN = 2;
		
		public int getRowCount(){
			return associationRules.size();
		}

		public int getColumnCount(){
			return 3;
		}

		public String getColumnName(int columnIndex){
			switch(columnIndex){
			case DOC_RULE_COLUMN: return "Rule";
			case DOC_CONFIDENCE_COLUMN: return "Confidence";
			case DOC_SUPPORT_COLUMN: return "Support";
			default: return "?";
			}
		}

		public Class getColumnClass(int columnIndex){
			switch(columnIndex){
			case DOC_RULE_COLUMN: return String.class;
			case DOC_CONFIDENCE_COLUMN: return String.class;
			case DOC_SUPPORT_COLUMN: return Integer.class;
			default: return Object.class;
			}
		}

		public boolean isCellEditable(int rowIndex, int columnIndex){
			return false;
		}
		
		
		public Object getValueAt(int rowIndex, int columnIndex){
			Rule rule = associationRules.get(rowIndex);
			Double conf = new Double(rule.getConfidence());
			Integer supp = new Integer(rule.getSupport());
			switch(columnIndex){
			case DOC_RULE_COLUMN: return new String(rule.leftHandSideToString() + " -> " + rule.getRightHandSide());
			case DOC_CONFIDENCE_COLUMN: return new String(conf.toString());
			case DOC_SUPPORT_COLUMN: return supp;
			default: return null;
			}
		}
	}

}
