package idi.gate.associationrules;

import idi.gate.associationrules.generator.AssociationRulesGenerator;
import idi.gate.associationrules.generator.FrequentItemSetsGenerator;
import idi.gate.associationrules.rule.Rule;
import idi.gate.index.Document;
import idi.gate.index.Index;
import idi.gate.index.StopWords;
import idi.gate.ontology.concept.Concept;
import idi.gate.ontology.concept.EnvironmentGenerator;
import idi.gate.ontology.concept.ConceptIsAExtractor;
import idi.gate.ontology.concept.ParentFinder;
import idi.gate.ontology.writer.OntologyWriter;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.TreeMap;

import gate.*;
import gate.persist.PersistenceException;
import gate.persist.SerialDataStore;
import gate.util.InvalidOffsetException;

import gate.creole.ResourceInstantiationException;
import gate.util.GateException;

public class Program {

	private static final String DS_DIR = "file:/C:/kvarv/Diplom/GATE/datastore/pmbok";
	
	private Corpus corpus;
	private Index index;
	private FrequentItemSetsGenerator frequentItemSetsGenerator;
	private Configuration config;
	private StopWords stopWords;
	private ArrayList<String> stopWordList;
	long start;
	long end;
	
	public Program() {
		index = Index.getInstance();
		try {
			stopWords = new StopWords(new URL("file:/C:/Documents and Settings/kvarv.WIN-NTNU-NO/Desktop/stopwords.txt"));
			stopWordList = stopWords.getStopWordList();
		} catch (IOException e1) {
			e1.printStackTrace();
		}
		config = Configuration.getInstance();
		config.setAnnotationName("NounPhrase");
		config.setInputAnnotationSetName("");
		config.setMinimumSupport(new Float(5.0).floatValue());
		config.setMiniumConfidence(new Float(75.0).floatValue());
		config.setNofFrequentTerms(45);
		config.setStopWordList(stopWordList);
		start = System.currentTimeMillis();
		Gate.setGateHome(new File("C:\\Program Files\\GATE-3.1\\"));
		try {
			Gate.init();
			getDocuments();
		} catch (GateException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		System.out.println("Starting computations...");
		index.buildIndex();
		
		frequentItemSetsGenerator = new FrequentItemSetsGenerator();
		System.out.println("\n *** Building frequent item set ***\n");
		
		ArrayList<TreeMap<String, ArrayList<Integer>>> fis =  frequentItemSetsGenerator.getListOfFrequentItemSets();
		AssociationRulesGenerator associationRulesGenerator = new AssociationRulesGenerator(fis);
		associationRulesGenerator.buildAssociations();
		end = System.currentTimeMillis();
		System.out.println("Total tid i sek: " + NumberFormat.getInstance().format((double)(end - start)/1000));
		ArrayList<Rule> rules = associationRulesGenerator.getAssociationRules();
		
		/*
		RuleSearcher searcher = new RuleSearcher(rules);
		try {
			searcher.searchTest();
		} catch (IOException e) {
			e.printStackTrace();
		}*/
		
		EnvironmentGenerator environmentGenerator = new EnvironmentGenerator(rules);
		ArrayList<Concept> concepts = environmentGenerator.generateConceptsAndEnvironments();
		System.out.println("Antall konsepter: " + concepts.size());
		ConceptIsAExtractor conceptIsAExtractor = new ConceptIsAExtractor(concepts);
		conceptIsAExtractor.extractConceptIsA();
		OntologyWriter ontologyWriter;
		try {
			ontologyWriter = new OntologyWriter(new URL("file:/C:/Documents and Settings/kvarv.WIN-NTNU-NO/Desktop/pmbok.owl"), concepts);
			ontologyWriter.writeOntology();
		} catch (MalformedURLException e) {
			e.printStackTrace();
		}
		//ParentFinder parentFinder = new ParentFinder(rules, concepts);
		//parentFinder.findCandidateParents();
		//parentFinder.isCandidateParent();
		//parentFinder.dfs();
		//parentFinder.printParents();

	}
	
	
	

	private Corpus getCorpus() throws PersistenceException, 
		ResourceInstantiationException{
		
		SerialDataStore sds = new SerialDataStore(DS_DIR);
		sds.open();

		List corpusIds = sds.getLrIds("gate.corpora.SerialCorpusImpl");

		FeatureMap features = Factory.newFeatureMap();  
		features.put(DataStore.LR_ID_FEATURE_NAME, corpusIds.get(0));  
		features.put(DataStore.DATASTORE_FEATURE_NAME, sds); 
		Corpus corpus = (Corpus) Factory.createResource("gate.corpora.SerialCorpusImpl", features);
		sds.close();
			
		return corpus;
	}
	
	private void getDocuments() throws PersistenceException, 
		ResourceInstantiationException, InvalidOffsetException {	
		corpus = getCorpus();
		Iterator iter = corpus.iterator();
		int counter = 0;
		while(iter.hasNext()){
			System.out.println("Retrieving document");
			gate.Document gateDoc = (gate.Document) iter.next();
			Document doc = new Document(gateDoc, counter);
			counter++;
		}
	}

	public static void main(String[] args) {
		Program program = new Program();
	}

}
