package idi.gate.associationrules.generator;


import idi.gate.associationrules.Configuration;
import idi.gate.associationrules.rule.Rule;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.TreeMap;
import java.util.Map.Entry;

public class AssociationRulesGenerator {
	
	private Configuration config;
	private ArrayList<Rule> associationRules;
	private ArrayList<TreeMap<String, ArrayList<Integer>>> listOfFrequentItemSets;
	private float miniumConfidence;
	
	public AssociationRulesGenerator (ArrayList<TreeMap<String, ArrayList<Integer>>> fis) {
		config = Configuration.getInstance();
		listOfFrequentItemSets = fis;
		miniumConfidence = config.getMiniumConfidence()/100;
		System.out.println("## minimumConfidence is: " + miniumConfidence + " ##");
	}
	
	public void buildAssociations() {
		associationRules = new ArrayList<Rule>();
		
		for (int i = listOfFrequentItemSets.size()-1; i > 0; i--) {
			TreeMap<String, ArrayList<Integer>> currentMap = listOfFrequentItemSets.get(i);
			
			ArrayList<Rule> generatedRules = generateAssociationsFromItemSets(currentMap);
			associationRules.addAll(generatedRules);			
		}
		//printAssociationRules();
		System.out.println("Number of rules: " + associationRules.size());	
	}

	private void printAssociationRules() {
		int counter = 0;
		for (int i = 0; i < associationRules.size(); i++) {
			String [] lh = associationRules.get(i).getLeftHandSide();
			if (lh.length == 1) {
			System.out.println(associationRules.get(i).leftHandSideToString() + " -> " + 
					associationRules.get(i).getRightHandSide() + " # confidence: " + 
					associationRules.get(i).getConfidence() + ", # support: " + 
					associationRules.get(i).getSupport());
			counter++;
			}
		}
		System.out.println("Antall 1-til-1: "+counter);
	}
	
	private ArrayList<Rule> generateAssociationsFromItemSets(TreeMap<String, ArrayList<Integer>> 
			frquentItemSets) {
		
		ArrayList<Rule> currentRules = new ArrayList<Rule>();

		Iterator iterator = frquentItemSets.entrySet().iterator();
		while(iterator.hasNext()){
			Entry<String, ArrayList<Integer>> entry = (Entry<String, ArrayList<Integer>>) iterator.next();
			String itemSet = entry.getKey();
			ArrayList<Integer> supportList = entry.getValue();
			int support = supportList.size();
			String [] itemSetArray = itemSetToArray(itemSet);
			int passNumber = itemSetArray.length;

			for (int i = passNumber-1; i >= 0 ; i--) {
				String rightHandSide = itemSetArray[i];
				String [] leftHandSide = new String[passNumber-1];
				int counter = 0;
				for (int j = 0; j < passNumber; j++) {
					if (j != i) {
						leftHandSide [counter] = itemSetArray[j];
						counter++;
					}
				}
				Rule rule = new Rule(leftHandSide, rightHandSide);
				String leftHand = rule.leftHandSideToString();
				double confidence = getConfidence(support, leftHand, passNumber);
				rule.setConfidence(confidence);
				rule.setSupport(support);

				if (confidence >= miniumConfidence) { 
					currentRules.add(rule);
				}
			}
		}		
		return currentRules;
	}
	
	private double getConfidence(int support, String leftHand, int passNumber) {
		
		int previousPass = passNumber-1;
		
		TreeMap<String, ArrayList<Integer>> itemSets = listOfFrequentItemSets.get(previousPass-1);
		ArrayList<Integer> supportList = itemSets.get(leftHand);
		int supportLeftHand = supportList.size();
		double confidence = (double)support/supportLeftHand;
		
		return confidence;
	}
	
	private String [] itemSetToArray(String key) {
		String [] list = key.split(",");
		return list;
	}

	public ArrayList<Rule> getAssociationRules() {
		return associationRules;
	}
}
