package idi.gate.associationrules.generator;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.TreeMap;
import java.util.Map.Entry;
import java.lang.Math;

import idi.gate.associationrules.*;
import idi.gate.index.Index;

public class FrequentItemSetsGenerator {
	
	private Index index;
	private Configuration config;
	private float minimumSupport;
	private int nofDocuments;
	private TreeMap<String, ArrayList<Integer>> database;
	private ArrayList<TreeMap<String, ArrayList<Integer>>> listOfFrequentItemSets;
	private int pass;
	private boolean doneFindingItemSets;
	
	public FrequentItemSetsGenerator(){
		index = Index.getInstance();
		nofDocuments = index.getNofDocuments();
		config = Configuration.getInstance();
		minimumSupport = supportAsNofDocuments(config.getMinimumSupport());
		pass = 1;
		doneFindingItemSets = false;
		listOfFrequentItemSets = new ArrayList<TreeMap<String,ArrayList<Integer>>>();
		
	}
	
	public TreeMap<String, ArrayList<Integer>> generateTestData() {
		TreeMap<String, ArrayList<Integer>> test = new TreeMap<String, ArrayList<Integer>>();
		ArrayList<Integer> a = new ArrayList<Integer>();
		a.add(0);
		a.add(1);
		a.add(2);
		a.add(3);
		a.add(4);
		ArrayList<Integer> b = new ArrayList<Integer>();
		b.add(0);
		b.add(1);
		b.add(4);
		ArrayList<Integer> c = new ArrayList<Integer>();
		c.add(0);
		c.add(1);
		c.add(2);
		c.add(3);
		c.add(4);
		ArrayList<Integer> d = new ArrayList<Integer>();
		d.add(1);
		d.add(2);
		d.add(3);
		d.add(4);
		ArrayList<Integer> e = new ArrayList<Integer>();
		e.add(1);
		e.add(3);
		test.put("A", a);
		test.put("B", b);
		test.put("C", c);
		test.put("D", d);
		test.put("E", e);
		return test;
	}
	
	private int supportAsNofDocuments(float value) {
		float support = ((nofDocuments*value)/100);
		int supportRounded = Math.round(support);
		return supportRounded;
	}
	
	private void findFrequentItemSets() {
		findFirstFrequentItemSets();
		while(doneFindingItemSets == false) {
			generateFrequentItemSets();
		}
	}
	
	private void findFirstFrequentItemSets(){
		database = index.getDocumentIndex();
		//database = generateTestData();
		//nofDocuments = database.size();
		//minimumSupport = supportAsNofDocuments(config.getMinimumSupport());
		TreeMap<String, ArrayList<Integer>> frequentItemSets = new TreeMap<String, ArrayList<Integer>>();
		
		Iterator iterator = database.entrySet().iterator();
		while(iterator.hasNext()){
			Entry<String, ArrayList<Integer>> entry = (Entry<String, ArrayList<Integer>>) iterator.next();
			String key = entry.getKey();
			ArrayList<Integer> value = entry.getValue();
			if (value.size() >= minimumSupport) frequentItemSets.put(key, value);
		}
		listOfFrequentItemSets.add(frequentItemSets);
		pass++;
	}
	
	private void generateFrequentItemSets() {
		int nofMatchingItems = pass - 2;
		TreeMap<String, ArrayList<Integer>> previousFrequentItemSets = (TreeMap<String, ArrayList<Integer>>) 
																		listOfFrequentItemSets.get(pass-2);
		
		TreeMap<String, ArrayList<Integer>> currentFrequentItemSets = new TreeMap<String, ArrayList<Integer>>();
 		ArrayList<String> keys = getKeys(previousFrequentItemSets);
		
 		if (nofMatchingItems == 0) {
 			for (int i = 0; i < keys.size(); i++) {
 				String currentKey = keys.get(i);
 				for (int j= i+1; j < keys.size(); j++){
 					String nextKey = keys.get(j);
 					String newKey = joinItemSets(currentKey, nextKey);
 					ArrayList<Integer> supportList = getSupportList(previousFrequentItemSets, currentKey, nextKey);
 					if (supportList.size() >= minimumSupport) currentFrequentItemSets.put(newKey, supportList);
 				}
 			}		
 		}
 		else {
 			for (int i = 0; i < keys.size(); i++) {
 				String currentKey = keys.get(i);
 				String [] currentKeySplit = itemSetToArray(currentKey); 				
 				String itemToMatch = "";
 				
 				for (int x = 0; x < nofMatchingItems; x++) {
 					itemToMatch += currentKeySplit[x];
 				}
 				
 				for (int j= i+1; j < keys.size(); j++){
 					String nextKey = keys.get(j);
 					String [] nextKeySplit = itemSetToArray(nextKey);
 					String matchingItem = "";
 					String itemToAdd = nextKeySplit[nofMatchingItems];
 					
 	 				for (int y = 0; y < nofMatchingItems; y++) {
 	 					matchingItem += nextKeySplit[y];
 	 				}

 	 				if (itemToMatch.equals(matchingItem)) {
 	 					String newKey = joinItemSets(currentKey, itemToAdd);
 	 					ArrayList<Integer> supportList = getSupportList(previousFrequentItemSets, currentKey, nextKey);
 	 					if (supportList.size() >= minimumSupport) currentFrequentItemSets.put(newKey, supportList);
 	 				}
 				}
 			}
 		}
 		//printMap(currentFrequentItemSets);
 		if (currentFrequentItemSets.size() != 0) listOfFrequentItemSets.add(currentFrequentItemSets);
 		else doneFindingItemSets = true;
 		pass++;
	}
	
	private String [] itemSetToArray(String key) {
		String [] list = key.split(",");
		return list;
	}
	
	private String joinItemSets (String key1, String key2) {
		String newKey = key1 + "," + key2;
		return newKey;
	}
	
	
	private ArrayList<String> getKeys(TreeMap<String, ArrayList<Integer>> frequentItemSets) {
		ArrayList<String> keys = new ArrayList<String>();
		Iterator iterator = frequentItemSets.entrySet().iterator();
		while(iterator.hasNext()){
			Entry<String, ArrayList<Integer>> entry = (Entry<String, ArrayList<Integer>>) iterator.next();
			String key = entry.getKey();
			keys.add(key);
		}	
		return keys;
	}
	
	private ArrayList<Integer> getSupportList(TreeMap<String, ArrayList<Integer>> frequentItemSets, 
			String key1, String key2) {
		
		ArrayList<Integer> supportList = new ArrayList<Integer>();
		ArrayList<Integer> list1 = frequentItemSets.get(key1);
		ArrayList<Integer> list2 = frequentItemSets.get(key2);
		
		for(int i = 0; i < list1.size(); i++) {
			Integer docNr = list1.get(i);
			if (list2.contains(docNr)) supportList.add(docNr);
		}
		
		return supportList;
	}
	
	private void printMap(TreeMap<String, ArrayList<Integer>> map) {
		Iterator iterator = map.entrySet().iterator();
		while(iterator.hasNext()){
			Entry<String, ArrayList<Integer>> entry = (Entry<String, ArrayList<Integer>>) iterator.next();
			String key = entry.getKey();
			ArrayList<Integer> value = entry.getValue();
			System.out.println("- "+key);
			for (int i = 0; i < value.size(); i++) {
				System.out.println("   "+ value.get(i));
			}
		}
	}

	public ArrayList<TreeMap<String, ArrayList<Integer>>> getListOfFrequentItemSets() {
		findFrequentItemSets();
		System.out.println("## minimumSupport: "+minimumSupport+" documents.");
		return listOfFrequentItemSets;
	}
	

}
