package idi.gate.associationrules.rule;


import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;

public class RuleSearcher {
	
	private ArrayList<Rule> rules;
	private BufferedReader stdin =  new BufferedReader(new InputStreamReader( System.in ));
	
	public RuleSearcher(ArrayList<Rule> ruleList) {
		this.rules = ruleList;
		trimRules();
	}
	
	private void trimRules() {
		ArrayList<Rule> trimmed = new ArrayList<Rule>();
		for (Rule rule: rules) {
			if (!(rule.getLeftHandSide().length > 1)) {
				trimmed.add(rule);
			}
		}
		rules = trimmed;
	}

	
	public ArrayList<Rule> searchLeftHand(String needle) {
		ArrayList<Rule> result = new ArrayList<Rule>();
		for (Rule rule: rules) {
			String lh = rule.leftHandSideToString();
			if (lh.equals(needle)) result.add(rule);
		}
		return result;
	}
	
	public void searchRightHand(String needle) {
		ArrayList<Rule> result = new ArrayList<Rule>();
		for (Rule rule: rules) {
			String rh = rule.getRightHandSide();
			if (rh.equals(needle)) result.add(rule);
		}
		if (!result.isEmpty()) {
			for (Rule rule: result) {
				System.out.println(rule.toString());
			}
		}
		else System.out.println("Couldn't find any rules matching " + needle + ".");
	}
	
	public void searchLeftTest(String needle) {
		ArrayList<Rule> result = new ArrayList<Rule>();
		for (Rule rule: rules) {
			String lh = rule.leftHandSideToString();
			if (lh.equals(needle)) result.add(rule);
		}
		if (!result.isEmpty()) {
			for (Rule rule: result) {
				System.out.println(rule.toString());
			}
		}
		else System.out.println("Couldn't find any rules matching " + needle + ".");
	}
	
	public Rule searchLeftHandAndRightHand(String leftHand, String rightHand) {
		for (Rule rule: rules) {
			String lh = rule.leftHandSideToString();
			String rh = rule.getRightHandSide();
			if (lh.equals(leftHand) && rh.equals(rightHand)) return rule;
		}
		return null;	
	}
	
	public void search(String needle) {
		ArrayList<Rule> result = new ArrayList<Rule>();
		for (Rule rule: rules) {
			String lh = rule.leftHandSideToString();
			String rh = rule.getRightHandSide();
			if (rh.equals(needle) || lh.equals(needle)) result.add(rule);
		}
		if (!result.isEmpty()) {
			for (Rule rule: result) {
				System.out.println(rule.toString());
			}
		}
		else System.out.println("Couldn't find any rules matching " + needle + ".");
	}
	
	public void searchTest() throws IOException {
		System.out.print( "Types: l - lefthand, r -  righthand, a - any, exit - exit\n" );
		boolean running = true;
		while (running) {
			System.out.print( "Search type: " );
			String type = stdin.readLine();
			System.out.print( "Search for: " );
	
			if (type.equals("a")) {
				String needle = stdin.readLine();
				search(needle);
			}
			else if (type.equals("l")) {
				String needle = stdin.readLine();
				searchLeftTest(needle);
			}
			else if (type.equals("r")) {
				String needle = stdin.readLine();
				searchRightHand(needle);
			}
			else if (type.equals("exit")) running = false;
			else System.out.println("Invalid type.");
		}
	}

		
}
