package idi.gate.index;

import gate.*;
import gate.util.GateRuntimeException;
import gate.util.InvalidOffsetException;

import idi.gate.associationrules.Configuration;

import java.util.*;



public class DocumentStatistics {

	private gate.Document document;
	private int nofFrequentTerms;
	private String annotationName;
	private String inputAnnotationSetName;
	private Configuration config;
	ArrayList<String> frequentTermList;
	ArrayList<String> stopWordList;

	public DocumentStatistics(gate.Document doc) {
		document = doc;
		config = Configuration.getInstance();
		nofFrequentTerms = config.getNofFrequentTerms();
		annotationName = config.getAnnotationName();
		inputAnnotationSetName = config.getInputAnnotationSetName();
		stopWordList = config.getStopWordList();
		getFrequentTermsList();
	}
	
	private void getFrequentTermsList() {
		ArrayList<String> termList = getTerms();	
		HashMap<String, Double> termFrequencyMap = getUniqueTermsAndFrequency(termList);
		frequentTermList = sortAndGetFrequentTerms(termFrequencyMap, nofFrequentTerms);
		Factory.deleteResource(document);
		//printFrequentTerms();
	}
	
	private void printFrequentTerms() {
		for(int i= 0; i < frequentTermList.size(); i++) {
			System.out.println(frequentTermList.get(i));
		}
		System.out.println("\n\n");
	}

	private ArrayList<String> getTerms() {
		
		ArrayList<String> termList = new ArrayList<String>();

		if(inputAnnotationSetName != null && inputAnnotationSetName.equals("")) 
			inputAnnotationSetName = null;		
		
		AnnotationSet inputAnnotationSet = (inputAnnotationSetName == null) ? 
				document.getAnnotations() : document.getAnnotations(inputAnnotationSetName);
		
		AnnotationSet annotationSet = inputAnnotationSet.get(annotationName);

		if (annotationSet != null){
			Iterator it = annotationSet.iterator();
			Annotation currentAnnotation;

			while(it.hasNext()){
				currentAnnotation = (Annotation) it.next();
				String term = "";
				FeatureMap featureMap = currentAnnotation.getFeatures();
				if (featureMap.containsKey("lemma")) {
					term = (String) currentAnnotation.getFeatures().get("lemma");
				}
				else {
					try {
						term = document.getContent().getContent(currentAnnotation.getStartNode().getOffset(),
								currentAnnotation.getEndNode().getOffset()).toString();
					} catch (InvalidOffsetException e) {
						e.printStackTrace();
					}					
				}
				term = term.trim();
				termList.add(term.toLowerCase());
			}	
		}
		else{
			throw new GateRuntimeException("Couldn't find any annotations with name '" + annotationName + "'");
		}
		return termList;
		
	}

	private HashMap<String, Double> getUniqueTermsAndFrequency(ArrayList<String> termList) {
		
		HashMap<String, Double> termFrequencyMap = new HashMap<String, Double>();

		for (int i = 0; i < termList.size(); i++) {
			String term = termList.get(i);
			ArrayList<String> multiTerms = splitAndAddMultiWordTerms(term);
			for (String word: multiTerms) {
				if (!termFrequencyMap.containsKey(word)){
					termFrequencyMap.put(word, new Double(1));
				}
				else{
					Double count = termFrequencyMap.get(word);
					count++;
					termFrequencyMap.put(word, count);
				}
			}
		}
		return termFrequencyMap;
	}
	
	private ArrayList<String> sortAndGetFrequentTerms(HashMap<String, Double> termFrequencyMap, int number) {
		List<Map.Entry<String, Double>> list = new Vector<Map.Entry<String, Double>>(termFrequencyMap.entrySet());

		Collections.sort(list, new Comparator<Map.Entry<String, Double>>(){
			public int compare(Map.Entry<String, Double> entry, Map.Entry<String, Double> entry1) {
				return (entry.getValue().equals(entry1.getValue()) ? 0 : (entry.getValue() > entry1.getValue() ? -1 : 1));
			}
		});
		
		if (nofFrequentTerms > termFrequencyMap.size()) nofFrequentTerms = termFrequencyMap.size();
		
		termFrequencyMap.clear();
		
		ArrayList<String> frequentTermList = new ArrayList<String>();
		
		int counter = 0;
		for (Map.Entry<String, Double> entry: list) {	
			String key = entry.getKey();
			Double value = entry.getValue();
			termFrequencyMap.put(key, value);
			if (!stopWordList.isEmpty()) {
				if ((counter < nofFrequentTerms) && !(stopWordList.contains(key))) { 
					frequentTermList.add(key);
					counter++;
				}				
			}
			else {
				if (counter < nofFrequentTerms) { 
					frequentTermList.add(key);
					counter++;
				}				
			}

		}
		return frequentTermList;
	}
	
	private ArrayList<String> splitAndAddMultiWordTerms(String term) {
		//Relaxed
		ArrayList<String> termList = new ArrayList<String>();
		String[] termArray = term.split(" ");
		if(termArray.length == 1) {
			termList.add(termArray[0]);
			return termList;
		}
		else {
			ArrayList<String> terms = new ArrayList<String>();
			for (int i = 0; i < termArray.length; i++) terms.add(termArray[i]);
			for (int i = 0; i < terms.size(); i++) {
				for (int j = i; j < terms.size(); j++) {
					List<String> list = terms.subList(i, j+1);
					String combined = "";
					if(list.size() > 1) {
						for (String word: list) combined += word+" ";
						combined = combined.trim();
						termList.add(combined);
					}
				}
			}
			return termList;
		}
	}

	public ArrayList<String> getFrequentTerms() {
		return frequentTermList;
	}
	
}
