package idi.gate.index;

import java.util.*;
import java.util.Map.Entry;

public final class Index {
	
	private static Index instance;
	
	private ArrayList<Document> documentList;
	private TreeMap<String, ArrayList<Integer>> documentIndex;
	
	private Index() {
		documentList = new ArrayList<Document>();
		documentIndex = new TreeMap<String, ArrayList<Integer>>();
	}
	
	public static Index getInstance() {
		if (instance == null) instance = new Index();
		return instance;
	}
	
	public void registrerDocument(Document doc) {
		documentList.add(doc);
	}
	
	public void clearIndex() {
		documentList.clear();
		documentIndex.clear();
	}
	
	
	public void buildIndex() {
		System.out.println("Number of documents: " + documentList.size());
		for (int i = 0; i < documentList.size(); i++) {
			Document doc = documentList.get(i);
			Integer id = new Integer(doc.getDocumentId());
			DocumentStatistics docStats = doc.getDocStats();
			ArrayList<String> frequentTerms = docStats.getFrequentTerms();
			
			for (int j = 0; j < frequentTerms.size(); j++) {
				String key = frequentTerms.get(j);
				if (!documentIndex.containsKey(key)) {
					ArrayList<Integer> docIdsList = new ArrayList<Integer>();
					docIdsList.add(id);
					documentIndex.put(key, docIdsList);
				}
				else {
					ArrayList<Integer> tmpList = documentIndex.get(key);
					tmpList.add(id);
					documentIndex.put(key, tmpList);
				}
			}
		}
		//printIndex(documentIndex);
	}
	
	private void printIndex(TreeMap<String, ArrayList<Integer>> map) {
		Iterator iterator = map.entrySet().iterator();
		while(iterator.hasNext()){
			Entry<String, ArrayList<Integer>> entry = (Entry<String, ArrayList<Integer>>) iterator.next();
			String key = entry.getKey();
			ArrayList<Integer> value = entry.getValue();
				System.out.println("- "+key);
			for (int i = 0; i < value.size(); i++) {
				System.out.println("   "+ value.get(i));
			}
		}
	}
	
	public int getNofDocuments() {
		return documentList.size();
	}
	
	public ArrayList<Document> getDocuments() {
		return documentList;
	}
	
	public TreeMap<String, ArrayList<Integer>> getDocumentIndex() {
		return documentIndex;
	}

	
}
