package idi.gate.nlp.lemmatizer;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.TreeMap;

public class EnglishLemmatizer implements ILemmatizer{
	
	private IDictionary dictionary;
	
	public EnglishLemmatizer(URL file) {
		try {
			initialize(file);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	public void initialize(URL file) throws IOException {

		String fileURL = file.getFile();
		FileInputStream fis = new FileInputStream(fileURL);
		InputStreamReader isr = new InputStreamReader(fis);
		BufferedReader br = new BufferedReader(isr);
		
		String data;
		TreeMap<String, String> wordList = new TreeMap<String, String>();
	
		while ((data = br.readLine()) != null ) {
			String [] lemmaAndWords = data.split(" -> ");
			String lemma = lemmaAndWords[0];
			String [] words = lemmaAndWords[1].split(",");
			
			for (int i = 0; i < words.length; i++) {
				wordList.put(words[i], lemma);
			}
			wordList.put(lemma, lemma);
		}
		br.close();
	
		dictionary = new EnglishDictionary(wordList);
	}
	
	public String getLemma(String word) {
		try {
			Word wordObject = dictionary.lookUpWord(word);
			String lemma = wordObject.getLemma();
			return lemma;
		} catch (WordNotFoundException e) {
			return "Unknown";
		}
	}
	
	
	public static void main(String args[]) {
		try {
			EnglishLemmatizer lemmatizer = new EnglishLemmatizer(new URL("file:/C:/kvarv/Diplom/Lemmalists/lemmalists_english/lemmalist_english.txt"));
			String lemma = lemmatizer.getLemma("execution");
			System.out.println(lemma);
		} catch (MalformedURLException e) {
			e.printStackTrace();
		}
	}
}
