package idi.gate.nlp.lemmatizer;

import gate.Annotation;
import gate.AnnotationSet;
import gate.Factory;
import gate.FeatureMap;
import gate.ProcessingResource;
import gate.Resource;
import gate.creole.*;
import gate.util.GateException;
import gate.util.GateRuntimeException;
import gate.util.InvalidOffsetException;

import java.net.URL;
import java.util.Iterator;

@SuppressWarnings("serial")
public class EnglishLemmatizerWrapper extends AbstractLanguageAnalyser 
	implements ProcessingResource {
	
	private String inputAnnotationSetName;
	private String inputAnnotationName;
	private String outputFeatureName;
	private URL lemmaListFile;
	private EnglishLemmatizer lemmatizer;
	
	public void execute() throws ExecutionException {
		try {
			lemmatize();
		} catch (InvalidOffsetException e) {
			e.printStackTrace();
		} catch (GateException e) {
			e.printStackTrace();
		}
	}

	public Resource init() throws ResourceInstantiationException {
		return this;
	}
	
	public void reInit() throws ResourceInstantiationException {
		init();
	}
	
	public void lemmatize()
		throws GateException, InvalidOffsetException {
		
		if(lemmaListFile == null || lemmaListFile.equals("")) {
			throw new ResourceInstantiationException(
	        "The URL to the lemma list must be specified");
		}
		else {
			lemmatizer = new EnglishLemmatizer(lemmaListFile);
		}		
		if(inputAnnotationSetName != null && inputAnnotationSetName.equals("")) 
			inputAnnotationSetName = null;		
		
		AnnotationSet inputAnnotationSet = (inputAnnotationSetName == null) ? 
				document.getAnnotations() : document.getAnnotations(inputAnnotationSetName);	
				
		AnnotationSet annotationSet = inputAnnotationSet.get(inputAnnotationName);

		
		if (annotationSet != null){
			double nofTokens = (double) annotationSet.size();
			double rate = nofTokens/100;
			fireStatusChanged("Lemmatizing document...");
			fireProgressChanged(0);
			Iterator it = annotationSet.iterator();
			Annotation token;
			int counter = 1;
			while(it.hasNext()){
				token = (Annotation) it.next();
				FeatureMap map = token.getFeatures();
				String word = ""+document.getContent().getContent(token.getStartNode().getOffset(),
						token.getEndNode().getOffset());
				word = word.toLowerCase();
				String lemma = lemmatizer.getLemma(word);
				map.put(outputFeatureName, lemma);
				fireProgressChanged((int)(counter*rate));
				counter++;
			}//while
			fireProgressChanged(100);
			
		}
		else{
			throw new GateRuntimeException("Couldn't find any annotations with namm "+ inputAnnotationName +"! Please make sure that you" +
					" have run a tokenizer on the document/corpus");
		}
		fireProcessFinished();
	}

	
	// Getter and setter methods

	public String getInputAnnotationSetName() {
		return inputAnnotationSetName;
	}

	public void setInputAnnotationSetName(String inputAnnotationSetName) {
		this.inputAnnotationSetName = inputAnnotationSetName;
	}

	public String getOutputFeatureName() {
		return outputFeatureName;
	}

	public void setOutputFeatureName(String outputFeatureName) {
		this.outputFeatureName = outputFeatureName;
	}

	public URL getLemmaListFile() {
		return lemmaListFile;
	}

	public void setLemmaListFile(URL lemmaListFile) {
		this.lemmaListFile = lemmaListFile;
	}

	public String getInputAnnotationName() {
		return inputAnnotationName;
	}

	public void setInputAnnotationName(String inputAnnotationName) {
		this.inputAnnotationName = inputAnnotationName;
	}




}

