package idi.gate.nlp.nounphrase;

import gate.Annotation;
import gate.AnnotationSet;
import gate.Corpus;
import gate.DataStore;
import gate.Document;
import gate.Factory;
import gate.FeatureMap;
import gate.Gate;
import gate.ProcessingResource;
import gate.Resource;
import gate.creole.*;
import gate.persist.PersistenceException;
import gate.persist.SerialDataStore;
import gate.util.GateException;
import gate.util.GateRuntimeException;
import gate.util.InvalidOffsetException;
import gate.util.OffsetComparator;

import idi.gate.associationrules.Program;

import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

@SuppressWarnings("serial")
public class NounPhraseExtractor extends AbstractLanguageAnalyser 
	implements ProcessingResource {
	
	private String inputAnnotationSetName;
	private String ouputAnnotationSetName;
	private String outputFeatureStemName;
	private String outputFeatureLemmaName;
	private String annotationName;
	private AnnotationSet outputAnnotationSet;
	
	public void execute() throws ExecutionException {
		try {
			extractNounPhrase();
		} catch (InvalidOffsetException e) {
			e.printStackTrace();
		} catch (GateException e) {
			e.printStackTrace();
		}
	}

	public Resource init() throws ResourceInstantiationException {
		return this;
	}
	
	public void reInit() throws ResourceInstantiationException {
		init();
	}

	
	public void extractNounPhrase() 
		throws GateException, InvalidOffsetException {
		
		if(inputAnnotationSetName != null && inputAnnotationSetName.equals("")) 
			inputAnnotationSetName = null;		
		
		AnnotationSet inputAnnotationSet = (inputAnnotationSetName == null) ? 
				document.getAnnotations() : document.getAnnotations(inputAnnotationSetName);

		if(ouputAnnotationSetName != null && ouputAnnotationSetName.equals("")) 
			ouputAnnotationSetName = null;		
		
		outputAnnotationSet = (ouputAnnotationSetName == null) ? 
				document.getAnnotations() : document.getAnnotations(ouputAnnotationSetName);	
				
				
		Set annotTypesRequired = new HashSet();
		annotTypesRequired.add("Token");
		annotTypesRequired.add("SpaceToken");
		annotTypesRequired.add("Split");

		AnnotationSet requiredSet = inputAnnotationSet.get(annotTypesRequired);
			
		if (requiredSet != null){

			ArrayList<Annotation> annotationList = new ArrayList<Annotation>(requiredSet);
			Collections.sort(annotationList, new OffsetComparator());

			Iterator it = annotationList.iterator();
			Annotation currentAnnotation;

			ArrayList<Annotation> outputAnnotationList = new ArrayList<Annotation>();
			boolean insideNounPhrase = false;
			String lastTypeAdded = "";
			String lastWordAdded = "";

			while(it.hasNext()){
				currentAnnotation = (Annotation) it.next();
				String feature = null;
				String type = currentAnnotation.getType();
				String word = ""+document.getContent().getContent(currentAnnotation.getStartNode().getOffset(),
						currentAnnotation.getEndNode().getOffset());

				if(type.equals("Token")) {
					feature = (String) currentAnnotation.getFeatures().get("category");
					if ((feature.equals("NN") || feature.equals("NNP") || feature.equals("NNPS") || 
							feature.equals("NNS") || feature.equals("NP") || feature.equals("NPS")) &&
							!(word.equals("[") || word.equals("]"))){
						insideNounPhrase = true;
						lastTypeAdded = type;
						lastWordAdded = word;
						outputAnnotationList.add(currentAnnotation);
					}
					else if (insideNounPhrase == true){
						if (lastTypeAdded.equals("SpaceToken")) 
							outputAnnotationList.remove(outputAnnotationList.size()-1);
						if (!isPartOfWord(lastWordAdded))
							addAnnotation(outputAnnotationList);
						outputAnnotationList.clear();
						lastTypeAdded = "";
						insideNounPhrase = false;	
					}
				}
				else if(type.equals("SpaceToken") && insideNounPhrase == true) {
					if (lastTypeAdded.equals("SpaceToken")) {
						if (!isPartOfWord(lastWordAdded)) {
							insideNounPhrase = false;
							addAnnotation(outputAnnotationList);
							outputAnnotationList.clear();	
							lastTypeAdded = "";
						}
						else {
							outputAnnotationList.remove(outputAnnotationList.size()-1);
							lastTypeAdded = "";
						}
					}
					else { 
						outputAnnotationList.add(currentAnnotation);
						lastTypeAdded = type;
					}
				}
				else if(type.equals("Split") && insideNounPhrase == true) {
					if (lastTypeAdded.equals("SpaceToken")) 
						outputAnnotationList.remove(outputAnnotationList.size()-1);
					addAnnotation(outputAnnotationList);
					outputAnnotationList.clear();
					insideNounPhrase = false;
					lastTypeAdded = "";
				}
			}//while

		}
		else {
			throw new GateRuntimeException("No tokens to process! Please make sure that you" +
			" have run a tokenizer on the document/corpus");		
		}
		fireProcessFinished();
	}

	private boolean isPartOfWord(String str) {
		char ch = str.charAt(str.length()-1);
		char [] chars = new char[1]; 
		chars[0] = ch;
		String a = new String(chars);
		if (a.equals("")) return true;
		else return false;
	}

	public void addAnnotation(ArrayList<Annotation> outputAnnotationList) 
		throws InvalidOffsetException {
		Annotation startAnnotation = outputAnnotationList.get(0);
		Annotation endAnnotation = outputAnnotationList.get(outputAnnotationList.size()-1);
		Long start = startAnnotation.getStartNode().getOffset();
		Long end = endAnnotation.getEndNode().getOffset();
		FeatureMap features = Factory.newFeatureMap();
		
		if (!outputFeatureStemName.equals("")) {
			String stemmedVersion = "";
			for (int i = 0; i < outputAnnotationList.size(); i++) {
				Annotation currentAnnotation = (Annotation) outputAnnotationList.get(i);
				String content = ""+ document.getContent().getContent(currentAnnotation.getStartNode().getOffset(),
						currentAnnotation.getEndNode().getOffset());
				String type = currentAnnotation.getType();
				FeatureMap featureMap = currentAnnotation.getFeatures();
				if (type.equals("Token")) {
					if(featureMap.containsKey("stem")) {
						stemmedVersion += (String) currentAnnotation.getFeatures().get("stem");
					}
					else {
						throw new GateRuntimeException("Can't find any features named stem! Please make sure that you" +
						" have run a stemmer on the document/corpus");							
					}
				}
				else stemmedVersion += content;
			}
			features.put(outputFeatureStemName, stemmedVersion);
		}
		if (!outputFeatureLemmaName.equals("")) {
			String lemmaVersion = "";
			for (int i = 0; i < outputAnnotationList.size(); i++) {
				Annotation currentAnnotation = (Annotation) outputAnnotationList.get(i);
				String content = ""+ document.getContent().getContent(currentAnnotation.getStartNode().getOffset(),
						currentAnnotation.getEndNode().getOffset());
				String type = currentAnnotation.getType();
				FeatureMap featureMap = currentAnnotation.getFeatures();
				if (type.equals("Token")) {
					if(featureMap.containsKey("lemma")) {
						String lemma = (String) currentAnnotation.getFeatures().get("lemma");
						if (lemma.equals("Unknown")) lemmaVersion += content;
						else lemmaVersion += (String) currentAnnotation.getFeatures().get("lemma");
						
					}
					else {
						throw new GateRuntimeException("Can't find any features named lemma! Please make sure that you" +
						" have run a lemmatizer on the document/corpus");							
					}
				}
				else lemmaVersion += content;
			}
			features.put(outputFeatureLemmaName, lemmaVersion);
		}
		outputAnnotationSet.add(start, end, annotationName, features);		
	}
	
	// Getter and setter methods

	public String getInputAnnotationSetName() {
		return inputAnnotationSetName;
	}

	public void setInputAnnotationSetName(String inputAnnotationSetName) {
		this.inputAnnotationSetName = inputAnnotationSetName;
	}

	public String getOuputAnnotationSetName() {
		return ouputAnnotationSetName;
	}

	public void setOuputAnnotationSetName(String ouputAnnotationSetName) {
		this.ouputAnnotationSetName = ouputAnnotationSetName;
	}

	public String getAnnotationName() {
		return annotationName;
	}

	public void setAnnotationName(String annotationName) {
		this.annotationName = annotationName;
	}

	public String getOutputFeatureStemName() {
		return outputFeatureStemName;
	}

	public void setOutputFeatureStemName(String outputFeatureName) {
		this.outputFeatureStemName = outputFeatureName;
	}

	public String getOutputFeatureLemmaName() {
		return outputFeatureLemmaName;
	}

	public void setOutputFeatureLemmaName(String outputFeatureLemmaName) {
		this.outputFeatureLemmaName = outputFeatureLemmaName;
	}


}
