package idi.gate.nlp.singlenoun;

import gate.Annotation;
import gate.AnnotationSet;
import gate.Factory;
import gate.FeatureMap;
import gate.ProcessingResource;
import gate.Resource;
import gate.creole.*;
import gate.util.GateException;
import gate.util.GateRuntimeException;
import gate.util.InvalidOffsetException;

import java.util.Iterator;

@SuppressWarnings("serial")
public class NounExtractor extends AbstractLanguageAnalyser 
	implements ProcessingResource {
	
	private String inputAnnotationSetName;
	private String ouputAnnotationSetName;
	private String outputFeatureName;
	private String annotationName;
	private AnnotationSet outputAnnotationSet;
	
	public void execute() throws ExecutionException {
		try {
			extractNouns();
		} catch (InvalidOffsetException e) {
			e.printStackTrace();
		} catch (GateException e) {
			e.printStackTrace();
		}
	}

	public Resource init() throws ResourceInstantiationException {
		return this;
	}
	
	public void extractNouns()
		throws GateException, InvalidOffsetException {
		
		if(inputAnnotationSetName != null && inputAnnotationSetName.equals("")) 
			inputAnnotationSetName = null;		
		
		AnnotationSet inputAnnotationSet = (inputAnnotationSetName == null) ? 
				document.getAnnotations() : document.getAnnotations(inputAnnotationSetName);

		if(ouputAnnotationSetName != null && ouputAnnotationSetName.equals("")) 
			ouputAnnotationSetName = null;		
		
		outputAnnotationSet = (ouputAnnotationSetName == null) ? 
				document.getAnnotations() : document.getAnnotations(ouputAnnotationSetName);				
		
				
		AnnotationSet tokenAnnotationSet = inputAnnotationSet.get("Token");
		
		if (tokenAnnotationSet != null){
			Iterator it = tokenAnnotationSet.iterator();
			Annotation token;
			
			while(it.hasNext()){
				token = (Annotation) it.next();
				String type = (String) token.getFeatures().get("category");
				if(type != null){
					if (type.equals("NN") || type.equals("NNP") || type.equals("NNPS") || 
							type.equals("NNS") || type.equals("NP") || type.equals("NPS")){
						addAnnotation(token, type);
					}
				}
				else{
					throw new GateRuntimeException("The feature 'category' couldn't be found." +
							" Please make sure that you have run a POS-tagger on the document/corpus");					
				}
			}//while
		}
		else{
			throw new GateRuntimeException("No tokens to process! Please make sure that you" +
					" have run a tokenizer on the document/corpus");
		}
		fireProcessFinished();
	}
	
	public void addAnnotation(Annotation token, String featureValue) throws InvalidOffsetException {
		Long start = token.getStartNode().getOffset();
		Long end = token.getEndNode().getOffset();
		FeatureMap nounFeatures = Factory.newFeatureMap();
		nounFeatures.put(outputFeatureName, featureValue);
		outputAnnotationSet.add(start, end, annotationName, nounFeatures);		
	}
	
	
	// Getter and setter methods

	public String getInputAnnotationSetName() {
		return inputAnnotationSetName;
	}

	public void setInputAnnotationSetName(String inputAnnotationSetName) {
		this.inputAnnotationSetName = inputAnnotationSetName;
	}

	public String getOuputAnnotationSetName() {
		return ouputAnnotationSetName;
	}

	public void setOuputAnnotationSetName(String ouputAnnotationSetName) {
		this.ouputAnnotationSetName = ouputAnnotationSetName;
	}

	public String getAnnotationName() {
		return annotationName;
	}

	public void setAnnotationName(String annotationName) {
		this.annotationName = annotationName;
	}

	public String getOutputFeatureName() {
		return outputFeatureName;
	}

	public void setOutputFeatureName(String outputFeatureName) {
		this.outputFeatureName = outputFeatureName;
	}




}
