package idi.gate.ontology.concept;

import java.util.ArrayList;

public class Concept {
	
	private String term;
	private Concept superConcept;
	private ArrayList<Concept> subConcepts;
	private ArrayList<String> environment;
	private Concept parent;
	private ArrayList<Concept> candidateParents;
	private boolean isParentCandidate;
	private boolean visited;
	
	public Concept(String term) {
		this.term = term;
		subConcepts = new ArrayList<Concept>();
		candidateParents = new ArrayList<Concept>();
		isParentCandidate = false;
		visited = false;
		parent = null;
	}
	
	public Concept(String term, ArrayList<String> environment) {
		this.term = term;
		this.environment = environment;
		subConcepts = new ArrayList<Concept>();
		candidateParents = new ArrayList<Concept>();
		isParentCandidate = false;
		visited = false;
		parent = null;
	}



	public String getTerm() {
		return term;
	}

	public ArrayList<String> getEnvironment() {
		return environment;
	}
	
	public String environmentToString() {
		String envX = "";
		for (int i = 0; i < environment.size(); i++) {
			envX += environment.get(i);
			if (i < environment.size()-1) envX += ", ";
			if (i % 7 == 0 && i != 0 && environment.size() != 8) envX += "\n";
		}
		return "Environment("+term +") = {" + envX+"}";
	}
	
	public boolean hasSuperConcept() {
		return superConcept != null;
	}

	public Concept getParent() {
		return parent;
	}

	public void setParent(Concept parent) {
		this.parent = parent;
	}
	
	public ArrayList<Concept> getCandidateParents() {
		return candidateParents;
	}
	
	public void addCandidateParent(Concept concept){
		candidateParents.add(concept);
	}

	public boolean isParentCandidate() {
		return isParentCandidate;
	}

	public void setIsParentCandidate(boolean isParentCandidate) {
		this.isParentCandidate = isParentCandidate;
	}
	
	public Concept getSuperConcept() {
		return superConcept;
	}

	public void setSuperConcept(Concept parent) {
		this.superConcept = parent;
	}

	public ArrayList<Concept> getSubConcepts() {
		return subConcepts;
	}

	public void addSubConcept(Concept concept) {
		subConcepts.add(concept);
	}
	public boolean hasSubConcept(Concept concept) {
		if (subConcepts.contains(concept)) return true;
		return false;
	}
	public boolean hasSubConcepts() {
		if (!subConcepts.isEmpty()) return true;
		return false;
	}

	public boolean isVisited() {
		return visited;
	}

	public void setVisited(boolean visited) {
		this.visited = visited;
	}


}
