package idi.gate.ontology.concept;


import idi.gate.ontology.writer.OntologyWriter;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.TreeMap;

public class ConceptIsAExtractor {
	private ArrayList<Concept> concepts;
	private TreeMap<String, Concept> conceptToTermMapping;

	public ConceptIsAExtractor(ArrayList<Concept> concepts) {
		this.concepts = concepts;
		mapConceptToTerm();
	}

	private void mapConceptToTerm() {
		conceptToTermMapping = new TreeMap<String, Concept>();
		for (Concept concept : concepts) {
			conceptToTermMapping.put(concept.getTerm(), concept);
		}
	}

	public void extractConceptIsA() {
		for (Concept concept : concepts) {
			if(!concept.hasSuperConcept()) {
				String term = concept.getTerm();
				if (isMultiTerm(term)) {
					ArrayList<String> terms = splitConcept(term);
					Concept superConcept = null;
					for (String currentTerm : terms) {
						Concept currentConcept = getConcept(currentTerm);
						if (currentConcept != null) {
							if(superConcept != null) {
								currentConcept.setSuperConcept(superConcept);
								if (!superConcept.hasSubConcept(currentConcept))
									superConcept.addSubConcept(currentConcept);
								//System.out.println(currentTerm +" isA "+ superConcept.getTerm());
								superConcept = currentConcept;
							}
							else superConcept = currentConcept;
						}
					}
				}
			}
		}
		printConcepts();
	}

	private void printConcepts() {

		for (Concept concept : concepts) {
			/*
			String term = concept.getTerm();
			ArrayList<Concept> subConcepts = concept.getSubConcepts();

			if (!subConcepts.isEmpty()) {
				System.out.println(term + ": ");
				for (Concept subConcept : subConcepts) {
					System.out.println("  "+subConcept.getTerm());		
				}
			}*/
			
			findRoot(concept);

		}
	}

	private void buildSubConceptHierarchy(Concept concept) {
		String term = concept.getTerm();
		term = term.replaceAll(" ", "_");
		ArrayList<Concept> subConcepts = concept.getSubConcepts();

		if (!subConcepts.isEmpty()) {
			for (Concept subConcept : subConcepts) {
				String subTerm = subConcept.getTerm();
				subTerm = subTerm.replaceAll(" ", "_");
				String subClass = "<owl:Class rdf:ID=\""+subTerm+"\">\n" +
				//"<rdfs:subClassOf rdf:resource=\"#"+subConcept.getSuperConcept().getTerm()+"\"/>\n" +
			    "  <rdfs:subClassOf>\n" +
			    "    <owl:Class rdf:ID=\""+term+"\"/>\n" +
			    "  </rdfs:subClassOf>\n" +
				"</owl:Class>\n";
				//System.out.println(subClass);
				buildSubConceptHierarchy(subConcept);		
			}
		}

	}
	
	private void findRoot(Concept concept) {
		String term = concept.getTerm();
		term = term.replaceAll(" ", "_");
		if (!concept.hasSuperConcept()) {
			//System.out.println("<owl:Class rdf:ID=\""+term+"\"/>\n");
			buildSubConceptHierarchy(concept);
		}
	}

	private Concept getConcept(String term) {
		if (isConcept(term)) return conceptToTermMapping.get(term);
		return null;
	}

	private boolean isConcept(String term) {
		if (conceptToTermMapping.containsKey(term)) return true;
		return false;
	}

	private boolean isMultiTerm(String term) {
		String[] termArray = term.split(" ");
		if (termArray.length > 1) return true;
		return false;
	}

	private static ArrayList<String> splitConcept(String term) {
		ArrayList<String> termList = new ArrayList<String>();
		String[] termArray = term.split(" ");

		ArrayList<String> terms = new ArrayList<String>();
		for (String t: termArray) terms.add(t);
		for (int i = terms.size()-1; i >= 0 ; i--) {
			String combined = "";
			for (int j = i; j < terms.size(); j++) {
				combined += terms.get(j)+" ";
			}
			combined = combined.trim();
			termList.add(combined);
		}
		return termList;
	}

	public static void main(String[] args) throws MalformedURLException {
		ArrayList<Concept> concepts = new ArrayList<Concept>();
		concepts.add(new Concept("project management team"));
		concepts.add(new Concept("management team"));
		concepts.add(new Concept("team"));
		concepts.add(new Concept("team management"));
		concepts.add(new Concept("management"));
		concepts.add(new Concept("project team"));
		concepts.add(new Concept("project"));

		ConceptIsAExtractor cIsAE = new ConceptIsAExtractor(concepts);
		cIsAE.extractConceptIsA();
		OntologyWriter ontologyWriter = new OntologyWriter(new URL("file:/C:/Documents and Settings/kvarv.WIN-NTNU-NO/Desktop/pmbok.owl"), concepts);
		ontologyWriter.writeOntology();

	}
}
