package idi.gate.ontology.concept;

import idi.gate.associationrules.rule.Rule;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.TreeMap;
import java.util.Map.Entry;

public class EnvironmentGenerator {
	private ArrayList<Rule> rules;
	private TreeMap<String,ArrayList<String>> termsAndEnvironment;

	
	public EnvironmentGenerator(ArrayList<Rule> ruleList) {
		rules = ruleList;
		termsAndEnvironment = new TreeMap<String, ArrayList<String>>();
	}
	
	/*
	private void trimRules() {
		ArrayList<Rule> trimmed = new ArrayList<Rule>();
		for (Rule rule: rules) {
			if (!(rule.getLeftHandSide().length > 1))
				trimmed.add(rule);
		}
		rules = trimmed;
	}	*/
	
	public ArrayList<Concept> generateConceptsAndEnvironments() {
		for (Rule rule: rules) {
			String[] leftHand = rule.getLeftHandSide();
			String rightHand = rule.getRightHandSide();

			addLeftHand(leftHand, rightHand);
			addRightHand(leftHand, rightHand);
		}
		printEnvironmentUgly(termsAndEnvironment);
		ArrayList<Concept> concepts = createConcepts(termsAndEnvironment);
		return concepts;
	}
	
	private void addRightHand(String[] leftHand, String rightHand) {
		ArrayList<String> envRightHand;
		if(!termsAndEnvironment.containsKey(rightHand)) {
			envRightHand = new ArrayList<String>();
			for (int i = 0; i < leftHand.length; i++) {
				envRightHand.add(leftHand[i]);
			}
			termsAndEnvironment.put(rightHand, envRightHand);
		}
		else {
			envRightHand = termsAndEnvironment.get(rightHand);
			for (int i = 0; i < leftHand.length; i++) {
				if (!envRightHand.contains(leftHand[i])) envRightHand.add(leftHand[i]);
			}
			termsAndEnvironment.put(rightHand, envRightHand);
		}
	}
	private void addLeftHand(String[] leftHand, String rightHand) {
		for (int i = 0; i < leftHand.length; i++) {
			String term = leftHand[i];
			ArrayList<String> envLeftHand;
			if(!termsAndEnvironment.containsKey(term)) {
				envLeftHand = new ArrayList<String>();
				envLeftHand.add(rightHand);
				for (int j = 0; j < leftHand.length; j++) {
					if (j != i) {
						envLeftHand.add(leftHand[j]);
					}
				}

				termsAndEnvironment.put(term, envLeftHand);
			}
			else {
				envLeftHand = termsAndEnvironment.get(term);
				if (!envLeftHand.contains(rightHand)) envLeftHand.add(rightHand);
				for (int j = 0; j < leftHand.length; j++) {
					if (j != i) {
						if (!envLeftHand.contains(leftHand[j])) envLeftHand.add(leftHand[j]);
					}
				}			
				termsAndEnvironment.put(term, envLeftHand);
			}
		}
	}
	
	private ArrayList<Concept> createConcepts(TreeMap<String, ArrayList<String>> map) {
		ArrayList<Concept> concepts = new ArrayList<Concept>();
		Iterator iterator = map.entrySet().iterator();
		while(iterator.hasNext()){
			Entry<String, ArrayList<String>> entry = (Entry<String, ArrayList<String>>) iterator.next();
			String key = entry.getKey();
			ArrayList<String> value = entry.getValue();
			Concept concept = new Concept(key, value);
			System.out.println(concept.getTerm());
			concepts.add(concept);
		}
		return concepts;
	}
	
	
	private void printEnvironment(TreeMap<String, ArrayList<String>> map) {
		Iterator iterator = map.entrySet().iterator();
		System.out.println("Environments...\n");
		while(iterator.hasNext()){
			Entry<String, ArrayList<String>> entry = (Entry<String, ArrayList<String>>) iterator.next();
			String key = entry.getKey();
			ArrayList<String> value = entry.getValue();
			String envX = "";
			for (int k = 0; k < value.size(); k++) {
				envX += value.get(k);
				if (k < value.size()-1) envX += ", ";
				if (k % 7 == 0 && k != 0 && value.size() != 8) envX += "	\n";
			}
			System.out.println("Environment("+key +") = {" + envX+"}\n");
		}
	}
	
	private void printEnvironmentUgly(TreeMap<String, ArrayList<String>> map) {
		Iterator iterator = map.entrySet().iterator();
		System.out.println("Environments...\n");
		while(iterator.hasNext()){
			Entry<String, ArrayList<String>> entry = (Entry<String, ArrayList<String>>) iterator.next();
			String key = entry.getKey();
			ArrayList<String> value = entry.getValue();
			String envX = "";
			for (int k = 0; k < value.size(); k++) {
				envX += value.get(k);
				if (k < value.size()-1) envX += ",";
			}
			System.out.println(key + ","+envX);
		}
	}

} 
