package idi.gate.ontology.concept;

import idi.gate.associationrules.rule.Rule;
import idi.gate.associationrules.rule.RuleSearcher;

import java.lang.reflect.Array;
import java.util.ArrayList;
import java.util.TreeMap;

public class ParentFinder {
	
	private ArrayList<Rule> rules;
	private RuleSearcher searcher;
	private ArrayList<Concept> concepts;
	private TreeMap<String, Concept> conceptToTermMapping;
	
	public ParentFinder(ArrayList<Rule> rules, ArrayList<Concept> concepts) {
		this.rules = rules;
		trimRules();
		this.concepts = concepts;
		searcher = new RuleSearcher(this.rules);
		mapConceptToTerm();
	}
	
	private void mapConceptToTerm() {
		conceptToTermMapping = new TreeMap<String, Concept>();
		for (Concept concept : concepts) {
			conceptToTermMapping.put(concept.getTerm(), concept);
		}
	}
	
	private void trimRules() {
		ArrayList<Rule> trimmed = new ArrayList<Rule>();
		for (Rule rule: this.rules) {
			if (!(rule.getLeftHandSide().length > 1))
				trimmed.add(rule);
		}
		this.rules = trimmed;
	}
	
	public void findCandidateParents() {
		for (Concept concept : concepts) {
			findParent(concept);
		}
	}
	
	public void isCandidateParent() {
		System.out.println("\nDisse har ingen inngende:");
		for (Concept concept : concepts) {
			if (!concept.isParentCandidate()) System.out.println(concept.getTerm());
		}
	}
	
	public void printParents() {
		for (Concept concept : concepts) {
			if (concept.getParent() != null) System.out.println(concept.getTerm() + " has parent: " + concept.getParent().getTerm());
			else System.out.println(concept.getTerm() + " has no parent.");
		}
	}
	
	private void findParent(Concept concept){
		String term = concept.getTerm();
		ArrayList<Rule> result = searcher.searchLeftHand(term);
		ArrayList<String> candidateParents = new ArrayList<String>();
		if (!result.isEmpty()) {
			for (Rule rule : result) {
				double confidence = rule.getConfidence();
				String rightHand = rule.getRightHandSide();
				Rule reflectionRule = searcher.searchLeftHandAndRightHand(rightHand, term);
				if (reflectionRule != null) {
					double reflectionConfidence = reflectionRule.getConfidence();
					if (confidence > reflectionConfidence) {
						candidateParents.add(rightHand);
						Concept candidateParent = conceptToTermMapping.get(rightHand);
						concept.addCandidateParent(candidateParent);
						candidateParent.setIsParentCandidate(true);
					}
				}
				else {
					Concept candidateParent = conceptToTermMapping.get(rightHand);
					concept.addCandidateParent(candidateParent);
					candidateParent.setIsParentCandidate(true);
					candidateParents.add(rightHand);
				}
			}
			System.out.println("- " + concept.getTerm());
			for (Concept c: concept.getCandidateParents()) System.out.println("   "+c.getTerm());
		}
		else System.out.println("- "+term+" !!!Fant ingen regler. Denne er nok rotnoden");

	}
	
	public void dfs() {
		for (Concept concept : concepts) {
			if (!concept.isVisited()) dfsVisit(concept);
		}
	}
	
	private void resetVisited() {
		for (Concept concept : concepts) {
			concept.setVisited(false);
		}
	}
	
	private void dfsVisit(Concept concept) {
		concept.setVisited(true);
		for (Concept candidateParent : concept.getCandidateParents()) {
			if (!candidateParent.isVisited()) { 
				concept.setParent(candidateParent);
				dfsVisit(candidateParent);
			}
		}
	}
	
	private Concept recursive(Concept concept) {
		String term = concept.getTerm();
		ArrayList<Rule> result = searcher.searchLeftHand(term);
		ArrayList<String> candidateParents = new ArrayList<String>();
		if (!result.isEmpty()) {
			for (Rule rule : result) {
				double confidence = rule.getConfidence();
				String rightHand = rule.getRightHandSide();
				Rule reflectionRule = searcher.searchLeftHandAndRightHand(rightHand, term);
				if (reflectionRule != null) {
					double reflectionConfidence = reflectionRule.getConfidence();
					if (confidence > reflectionConfidence) candidateParents.add(rightHand);
				}
				else candidateParents.add(rightHand);
			}
			if (candidateParents.size() > 1) {
				for (String word : candidateParents) {
					Concept currentConcept = conceptToTermMapping.get(word);
					recursive(currentConcept);
				}
			}
			else {
				Concept parent = conceptToTermMapping.get(candidateParents.get(0));
				concept.setSuperConcept(parent);
			}
		}
		return concept;
	}
	
	/*
	public void findParents() {
		for (Rule rule: rules) {
			double confidence = rule.getConfidence();
			String [] leftHand = rule.getLeftHandSide();
			String rightHand = rule.getRightHandSide();
			ArrayList<Rule> result = searcher.searchLeftHandAndRightHand(rightHand, leftHand[0]);
			if(!result.isEmpty()) {
				for (Rule resultRule: result) {
					double resultConfidence = resultRule.getConfidence();
					if(confidence > resultConfidence) {
						Concept concept = new Concept(leftHand[0]);
						Concept parentConcept = new Concept(rightHand);
						concept.setParent(parentConcept);
						concepts.add(concept);
					}
					else {
						Concept concept = new Concept(rightHand);
						Concept parentConcept = new Concept(leftHand[0]);
						concept.setParent(parentConcept);
						concepts.add(concept);
					}
				}
			}
			else {
				Concept concept = new Concept(leftHand[0]);
				Concept parentConcept = new Concept(rightHand);
				concept.setParent(parentConcept);
				concepts.add(concept);
			}
		}
		for (Concept concept : concepts) {
			String term = concept.getTerm();
			Concept parent = concept.getParent();
			String parentTerm = parent.getTerm();
			System.out.println("Parent("+term+") = " + parentTerm);
		}
	}*/
	
}
