package idi.gate.ontology.writer;


import idi.gate.ontology.concept.Concept;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;

public class OntologyWriter {
	
	FileWriter fileWriter;
	BufferedWriter bufferedWriter;
	ArrayList<Concept> concepts;
	String fileName;
	int counter = 0;
	
	public OntologyWriter(URL file, ArrayList<Concept> concepts) {
		String fileURL = file.getFile();
		this.fileName = fileURL;
		this.concepts = concepts;

	}
	
	public void writeOntology() {
		try {
			fileWriter = new FileWriter(this.fileName);
			
			bufferedWriter = new BufferedWriter(fileWriter);
			writeOntologyHeader();
			writeClasses();
			writeOntologyFooter();
			bufferedWriter.close();
		} catch (IOException e) {
			e.printStackTrace();
		}

	}

	private void writeClasses() {
		for (Concept concept : concepts) {
			findRoot(concept);
		}
		System.out.println(counter);
	}
	
	private void buildSubConceptHierarchy(Concept concept) {
		String term = concept.getTerm();
		term = term.replaceAll(" ", "_");
		ArrayList<Concept> subConcepts = concept.getSubConcepts();

		if (!subConcepts.isEmpty()) {
			for (Concept subConcept : subConcepts) {
				String subTerm = subConcept.getTerm();
				subTerm = subTerm.replaceAll(" ", "_");
				String subClass = "<owl:Class rdf:ID=\""+subTerm+"\">\n" +
				"<rdfs:subClassOf rdf:resource=\"#"+subConcept.getSuperConcept().getTerm()+"\"/>\n" +
			    //"  <rdfs:subClassOf>\n" +
			    //"    <owl:Class rdf:ID=\""+term+"\"/>\n" +
			    //"  </rdfs:subClassOf>\n" +
				"</owl:Class>\n";
				counter++;
				try {
					bufferedWriter.write(subClass);
				} catch (IOException e) {
					e.printStackTrace();
				}
				buildSubConceptHierarchy(subConcept);		
			}
		}

	}

	
	private void findRoot(Concept concept) {
		String term = concept.getTerm();
		term = term.replaceAll(" ", "_");
		
		if (!concept.hasSuperConcept()) {
			counter++;
			try {
				bufferedWriter.write("<owl:Class rdf:ID=\""+term+"\"/>\n");
			} catch (IOException e) {
				e.printStackTrace();
			}
			buildSubConceptHierarchy(concept);
		}
	}
	
	private void writeOntologyHeader() {
		String header = "<?xml version=\"1.0\"?>\n"+
		"<rdf:RDF\n" +
	    "xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\"\n"+
	    "xmlns:xsd=\"http://www.w3.org/2001/XMLSchema#\"\n" +
	    "xmlns:rdfs=\"http://www.w3.org/2000/01/rdf-schema#\"\n" +
	    "xmlns:owl=\"http://www.w3.org/2002/07/owl#\"\n" +
	    "xmlns=\"http://www.owl-ontologies.com/unnamed.owl#\"\n" +
	    "xml:base=\"http://www.owl-ontologies.com/unnamed.owl\">\n" + 
		"<owl:Ontology rdf:about=\"\"/>\n";
		try {
			bufferedWriter.write(header);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	private void writeOntologyFooter() {
		String footer = "</rdf:RDF>";
		try {
			bufferedWriter.write(footer);
		} catch (IOException e) {
			e.printStackTrace();
		}		
	}

}
