package idi.testframework;

import idi.gate.nlp.lemmatizer.EnglishLemmatizer;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.TreeMap;
import java.util.Vector;

public class CosineSimilarityResults {
	StringTokenizer st;
	BufferedReader br;
	ArrayList<String> concepts;
	TreeMap<String, ArrayList<String>> relatedConceptsMap;
	EnglishLemmatizer lemmatizer;
	
	public CosineSimilarityResults (URL file) {
		concepts = new ArrayList<String>();
		relatedConceptsMap = new TreeMap<String, ArrayList<String>>();
		try {
			lemmatizer = new EnglishLemmatizer(new URL("file:/C:/kvarv/Diplom/Lemmalists/lemmalists_english/" +
					"lemmalist_english.txt"));
		} catch (MalformedURLException e1) {
			e1.printStackTrace();
		}
		try {
			initialize(file);
			readFile();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public void readFile() throws IOException {
		readConcepts();
		readValues();

	}
	
	public void initialize(URL file) throws IOException {

		String fileURL = file.getFile();
		FileInputStream fis = new FileInputStream(fileURL);
		InputStreamReader isr = new InputStreamReader(fis);
		br = new BufferedReader(isr);
		st = new StringTokenizer("");
		
	
	}
	
	private void readConcepts() throws IOException {
		String conceptsString = br.readLine();
		String [] array = conceptsString.split("	");
		for (String string : array) {
			string = string.toLowerCase();
			string = getLemma(string);
			concepts.add(string);
		}
	}
	
	private String getLemma(String concept) {
		String[] array = concept.split(" ");
		String lemma = "";
		for (String string : array) {
			String lem = lemmatizer.getLemma(string);
			if (!lem.equals("Unknown")) string = lem;
			lemma += string + " ";
		}
		lemma = lemma.trim();
		return lemma;
	}
	
	private void readValues() throws IOException {
		String values = br.readLine();
		while ((values = br.readLine()) != null ) {
			HashMap<String, Double> map = new HashMap<String, Double>();
			String [] array = values.split("	");
			String concept = array [0].toLowerCase();
			concept = getLemma(concept);
			for (int i = 1; i < array.length; i++) {
				String relatedConcept = concepts.get(i-1);
				String value = array[i];
				if (value.equals("01.01.2000")) value = "1.0";
				map.put(relatedConcept, new Double(value));
			}
			List<Map.Entry<String, Double>> list = new Vector<Map.Entry<String, Double>>(map.entrySet());

			Collections.sort(list, new Comparator<Map.Entry<String, Double>>(){
				public int compare(Map.Entry<String, Double> entry, Map.Entry<String, Double> entry1) {
					return (entry.getValue().equals(entry1.getValue()) ? 0 : (entry.getValue() > entry1.getValue() ? -1 : 1));
				}
			});
			
			ArrayList<String> concepts = new ArrayList<String>();
			int counter = 0;
			for (Map.Entry<String, Double> entry: list) {
				if (counter < 50) {
					String key = entry.getKey();
					concepts.add(key);
				}
				counter++;
			}
			relatedConceptsMap.put(concept, concepts);
		}
	}
	
	public void printRelatedConcepts() {
		if (relatedConceptsMap.containsKey("control")) {
			ArrayList<String> mordi = relatedConceptsMap.get("project plan");
			System.out.println("cost");
			for (String string : mordi) {
				System.out.println("  "+string);
			}
		}
	}
	
	public TreeMap<String, ArrayList<String>> getRelatedConceptsMap() {
		return relatedConceptsMap;
	}
	
	public static void main(String args[]) {
		try {
			CosineSimilarityResults cosineSimilarityResults = new CosineSimilarityResults(
				new URL("file:/C:/kvarv/Diplom/resultater/for real/" +
						"cosine similarity.txt"));
			cosineSimilarityResults.printRelatedConcepts();
		} catch (MalformedURLException e) {
			e.printStackTrace();
		}
	}

	
}
