package idi.testframework;

import idi.gate.nlp.lemmatizer.EnglishLemmatizer;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;

public class SemiOnotologyConcepts {
	private ArrayList<String> concepts;
	EnglishLemmatizer lemmatizer;
	
	public SemiOnotologyConcepts (URL file) {
		concepts = new ArrayList<String>();
		try {
			lemmatizer = new EnglishLemmatizer(new URL("file:/C:/kvarv/Diplom/Lemmalists/lemmalists_english/" +
					"lemmalist_english.txt"));
		} catch (MalformedURLException e1) {
			e1.printStackTrace();
		}
		try {
			initialize(file);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	public void initialize(URL file) throws IOException {

		String fileURL = file.getFile();
		FileInputStream fis = new FileInputStream(fileURL);
		InputStreamReader isr = new InputStreamReader(fis);
		BufferedReader br = new BufferedReader(isr);
		
		String data;
		while ((data = br.readLine()) != null ) {
			data = data.trim();
			findConcept(data);
		}
		br.close();
	
	}
	
	private void findConcept(String data) {
		int index = data.indexOf("ID=");
		if (index != -1) {
			String raw = data.substring(index+4);
			raw = raw.replaceAll("\">", "");
			raw = raw.replaceAll("\"/>", "");
			raw = raw.replaceAll("_", " ");
			String concept = raw.toLowerCase();
			concept = getLemma(concept);
			concepts.add(concept);
		}
	}
	
	private String getLemma(String concept) {
		String[] array = concept.split(" ");
		String lemma = "";
		for (String string : array) {
			String lem = lemmatizer.getLemma(string);
			if (!lem.equals("Unknown")) string = lem;
			lemma += string + " ";
		}
		lemma = lemma.trim();
		return lemma;
	}
	
	public void printConcepts() {
		for (String concept : concepts) {
			System.out.println(concept);
		}
		System.out.println("Antall: " + concepts.size());
	}
	
	public ArrayList<String> getConcepts() {
		return concepts;
	}
	
	public static void main(String args[]) {
		try {

			SemiOnotologyConcepts semiOnotologyConcepts = new SemiOnotologyConcepts(new URL("file:/C:/kvarv/Diplom/resultater/for real/" +
					"PMBOK_semiauto.owl"));
			semiOnotologyConcepts.printConcepts();
		} catch (MalformedURLException e) {
			e.printStackTrace();
		}
	}
}
