package idi.testframework;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.TreeMap;
import java.util.Vector;

public class SubjectEvaluationResults {

	StringTokenizer st;
	BufferedReader br;

	TreeMap<String, TreeMap<String, Double>> arRelatedConceptsValues;
	TreeMap<String, TreeMap<String, Double>> cosineRelatedConceptsValues;
	ArrayList<URL> files;
	
	public SubjectEvaluationResults (String subject) {
		files = new ArrayList<URL>();
		try {
			files.add(new URL("file:/C:/kvarv/Diplom/resultater/for real/associations/" +
			subject + "/cost.txt"));
			files.add(new URL("file:/C:/kvarv/Diplom/resultater/for real/associations/" +
					subject + "/control.txt"));
			files.add(new URL("file:/C:/kvarv/Diplom/resultater/for real/associations/" +
					subject + "/scope.txt"));
			files.add(new URL("file:/C:/kvarv/Diplom/resultater/for real/associations/" +
					subject + "/risk.txt"));
			files.add(new URL("file:/C:/kvarv/Diplom/resultater/for real/associations/" +
					subject + "/project_plan.txt"));
			files.add(new URL("file:/C:/kvarv/Diplom/resultater/for real/associations/" +
					subject + "/cos_cost.txt"));
			files.add(new URL("file:/C:/kvarv/Diplom/resultater/for real/associations/" +
					subject + "/cos_control.txt"));
			files.add(new URL("file:/C:/kvarv/Diplom/resultater/for real/associations/" +
					subject + "/cos_scope.txt"));
			files.add(new URL("file:/C:/kvarv/Diplom/resultater/for real/associations/" +
					subject + "/cos_risk.txt"));
			files.add(new URL("file:/C:/kvarv/Diplom/resultater/for real/associations/" +
					subject + "/cos_project_plan.txt"));
		} catch (MalformedURLException e) {
			e.printStackTrace();
		}
		arRelatedConceptsValues = new TreeMap<String, TreeMap<String,Double>>();
		cosineRelatedConceptsValues = new TreeMap<String, TreeMap<String,Double>>();
		try {
			readFiles();
		} catch (IOException e) {
			e.printStackTrace();
		}
		

	}

	public void readFiles() throws IOException {
		int counter = 0;
		for (URL file : files) {
			try {
				initialize(file);
			} catch (IOException e) {
				e.printStackTrace();
			}
			String concept = readConcept();
			if (counter < 5) readARValues(concept);
			else readCosineValues(concept);	
			counter++;
		}
	}
	
	public void initialize(URL file) throws IOException {

		String fileURL = file.getFile();
		FileInputStream fis = new FileInputStream(fileURL);
		InputStreamReader isr = new InputStreamReader(fis);
		br = new BufferedReader(isr);
		st = new StringTokenizer("");
		
	}
	
	private String readConcept() throws IOException {
		String concept = br.readLine();
		return concept;
	}
	
	private void readARValues(String concept) throws IOException {
		String mapping = "";
		TreeMap<String,Double> map = new TreeMap<String, Double>();
		while ((mapping = br.readLine()) != null ) {
			String input = mapping.trim();
			String [] array = input.split("				");
			map.put(array[0], new Double(array[1]));
		}
		arRelatedConceptsValues.put(concept, map);
	}
	
	private void readCosineValues(String concept) throws IOException {
		String mapping = "";
		TreeMap<String,Double> map = new TreeMap<String, Double>();
		while ((mapping = br.readLine()) != null ) {
			String input = mapping.trim();
			String [] array = input.split("				");
			map.put(array[0], new Double(array[1]));
		}
		cosineRelatedConceptsValues.put(concept, map);
	}
	
	public Double getArRelatedConceptScore(String chosenConcept, String relatedConcept) {
		TreeMap<String, Double> related = arRelatedConceptsValues.get(chosenConcept);
		return related.get(relatedConcept);
	}
	
	public Double getCosineRelatedConceptScore(String chosenConcept, String relatedConcept) {
		TreeMap<String, Double> related = cosineRelatedConceptsValues.get(chosenConcept);
		return related.get(relatedConcept);
	}
	
	public TreeMap<String, TreeMap<String, Double>> getCosineRelatedConceptsValues() {
		return cosineRelatedConceptsValues;
	}

	public TreeMap<String, TreeMap<String, Double>> getArRelatedConceptsValues() {
		return arRelatedConceptsValues;
	}
	
	public static void main(String args[]) {
		SubjectEvaluationResults subjectEvaluationResults = new SubjectEvaluationResults("claes");
		TreeMap<String, TreeMap<String, Double>> arRelatedConceptsValues = subjectEvaluationResults.getArRelatedConceptsValues();
		TreeMap<String, TreeMap<String, Double>> cosineRelatedConceptsValues = subjectEvaluationResults.getCosineRelatedConceptsValues();
		System.out.println(subjectEvaluationResults.getArRelatedConceptScore("control", "action"));
		System.out.println(subjectEvaluationResults.getCosineRelatedConceptScore("control", "change"));
		System.out.println(arRelatedConceptsValues);
		System.out.println(cosineRelatedConceptsValues);
	}


}
