package idi.testframework;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.TreeMap;
import java.util.Map.Entry;

public class TestCases {
	
	TreeMap<String, ArrayList<String>> onlyInCosine;
	TreeMap<String, ArrayList<String>> onlyInAR;
	TreeMap<String, TreeMap<String, String>> inBoth;
	
	SubjectEvaluationResults subject1;
	SubjectEvaluationResults subject2;
	SubjectEvaluationResults subject3;
	SubjectEvaluationResults subject4;
	ArrayList<SubjectEvaluationResults> subjects;
	
	public TestCases() {
		onlyInAR = new TreeMap<String, ArrayList<String>>();
		onlyInCosine = new TreeMap<String, ArrayList<String>>();
		inBoth = new TreeMap<String, TreeMap<String, String>>();
		subject1 = new SubjectEvaluationResults("claes");
		subject2 = new SubjectEvaluationResults("martin");
		subject3 = new SubjectEvaluationResults("hm");
		subject4 = new SubjectEvaluationResults("kv");
		subjects = new ArrayList<SubjectEvaluationResults>();
		subjects.add(subject1);
		subjects.add(subject2);
		subjects.add(subject3);
		subjects.add(subject4);
	}
	
	private int getArMean(String chosenConcept, String relatedConcept) {
		Double mean = 0.0;
		for (SubjectEvaluationResults subject : subjects) {
			Double value = subject.getArRelatedConceptScore(chosenConcept, relatedConcept);
			mean += value;
		}
		mean = mean/subjects.size();
		long result = Math.round(mean);
		//System.out.println((int) result);
		//System.out.println(mean);
		return (int) result;
	}
	
	private int getCosineMean(String chosenConcept, String relatedConcept) {
		Double mean = 0.0;
		for (SubjectEvaluationResults subject : subjects) {
			Double value = subject.getCosineRelatedConceptScore(chosenConcept, relatedConcept);
			mean += value;
		}
		mean = mean/subjects.size();
		long result = Math.round(mean);
		//System.out.println((int) result);
		//System.out.println(mean);
		return (int) result;
	}
	
	
	//	CONCEPTS
	public void checkConceptMatch(ArrayList<String> concepts, ArrayList<String> reference) {
		ArrayList<String> mismatchingConcepts = new ArrayList<String>();
		System.out.println("--- EVALUATING CONCEPTS ---\n");
		System.out.println("* Number of concepts found: " + concepts.size());
		System.out.println("* Number of concepts in ontology: " + reference.size()+"\n");
		int nofMatches = 0;
		int nofAbstractionMatches = 0;
		for (String refConcept : reference) {
			if (concepts.contains(refConcept)) {
				System.out.println("Match: " + refConcept+"\n");
				nofMatches++;
			}
			else {
				System.out.println("No match: " + refConcept);
				
				ArrayList<String> abstractions = getConceptAbtractions(refConcept);
				//System.out.println("Mismatched concept: "+ refConcept);
				for (int i = abstractions.size()-2; i >= 0; i--) {
					if (concepts.contains(abstractions.get(i))) {
						System.out.println("Abstraction match at level " +(abstractions.size()-(i+1))+ " of "
								+(abstractions.size()-1)+": "+ abstractions.get(i)+"\n");
						nofAbstractionMatches++;
						break;
					}
					else {
						if (i==0)System.out.println("No abstraction match: "+ abstractions.get(i)+"\n");
						else System.out.println("No abstraction match: "+ abstractions.get(i));
					}
				}
				mismatchingConcepts.add(refConcept);
			}
		}
		System.out.println("\n* Number of perfectly matching concepts: " + nofMatches);
		int nofMismathces = reference.size() - nofMatches;
		System.out.println("* Number of mismatches: " + nofMismathces);
		double percent = (100.0*nofMatches)/reference.size();
		System.out.println("* Match in percent: " + percent+"\n");
		System.out.println("\n* Number of matching abstractions: " + nofAbstractionMatches);
		double abPercent = (100.0*nofAbstractionMatches)/mismatchingConcepts.size();
		System.out.println("* Matching abstractions in percent of perfect mismatches: " + abPercent+"\n");
	}
	
	private ArrayList<String> getConceptAbtractions(String concept) {
		ArrayList<String> termList = new ArrayList<String>();
		String[] termArray = concept.split(" ");

		ArrayList<String> terms = new ArrayList<String>();
		for (String t: termArray) terms.add(t);
		for (int i = terms.size()-1; i >= 0 ; i--) {
			String combined = "";
			for (int j = i; j < terms.size(); j++) {
				combined += terms.get(j)+" ";
			}
			combined = combined.trim();
			termList.add(combined);
		}
		return termList;
	}	
	
	//	ASSOCIATIONS
	
	public void checkRelatedConceptMatch(ArrayList<String> chosenConcepts, 
			TreeMap<String, ArrayList<String>> concepts, TreeMap<String, ArrayList<String>> reference) {

		System.out.println("--- EVALUATING RELATIONS ---\n");
		for (String chosenConcept : chosenConcepts) {
			ArrayList<String> relatedConcepts = concepts.get(chosenConcept);
			ArrayList<String> cosineRelatedConcepts = reference.get(chosenConcept);
			System.out.println("***** " + chosenConcept.toUpperCase() + " *****");
			System.out.println("*Number of related concepts found: " + relatedConcepts.size());
			System.out.println("*Number of related concepts with cosine similarity: " + cosineRelatedConcepts.size()+"\n");
			
			int nofMatches = 0;
			int nofAbstractionMatches = 0;
			int nofMismatches = 0;
			TreeMap<String, String> both  = new TreeMap<String, String>();	
			
			for (String cosineRelatedConcept : cosineRelatedConcepts) {
				
				if (relatedConcepts.contains(cosineRelatedConcept) ||
						cosineRelatedConcept.equals(chosenConcept)) {
					System.out.println("Match: " + cosineRelatedConcept+"\n");
					nofMatches++;
					both.put(cosineRelatedConcept, cosineRelatedConcept);
				}
				else {
					nofMismatches++;
					System.out.println("No match: " + cosineRelatedConcept);
					ArrayList<String> abstractions = getConceptAbtractions(cosineRelatedConcept);
					for (int i = abstractions.size()-2; i >= 0; i--) {
						if (relatedConcepts.contains(abstractions.get(i))) {
							System.out.println("Abstraction match at level " +(abstractions.size()-(i+1))+ " of "
									+(abstractions.size()-1)+": "+ abstractions.get(i)+"\n");
							nofAbstractionMatches++;
							both.put(cosineRelatedConcept, abstractions.get(i));
							break;
						}
						else {
							if (i==0)System.out.println("No abstraction match: "+ abstractions.get(i)+"\n");
							else System.out.println("No abstraction match: "+ abstractions.get(i));
						}
					}
				}
			}
			inBoth.put(chosenConcept, both);
			System.out.println("\n*Number of perfectly matching related concepts: " + nofMatches);
			int nofMismathces = cosineRelatedConcepts.size() - nofMatches;
			System.out.println("*Number of mismatches: " + nofMismathces);
			double boost = (1.00*cosineRelatedConcepts.size())/relatedConcepts.size();
			double percent = (100.0*nofMatches)/cosineRelatedConcepts.size();
			System.out.println("*Match in percent: " + percent+"\n");
			//System.out.println("*Match in percent with boost: " + percent*boost+"\n");
			
			System.out.println("\n* Number of matching abstractions: " + nofAbstractionMatches);
			double abPercent = (100.0*nofAbstractionMatches)/nofMismatches;
			System.out.println("* Matching abstractions in percent of perfect mismatches: " + abPercent+"\n");			
		}
		//splitToGroups(inBoth, concepts, reference);
		System.out.println(onlyInAR);
		System.out.println(onlyInCosine);
		System.out.println(inBoth);
	}
	
	public void splitToGroups(TreeMap<String, TreeMap<String, String>> both, TreeMap<String, ArrayList<String>> concepts, 
			TreeMap<String, ArrayList<String>> reference) {
		
		Iterator iterator = both.entrySet().iterator();
		while(iterator.hasNext()){
			Entry<String, TreeMap<String, String>> entry = (Entry<String, TreeMap<String, String>>) iterator.next();
			String chosenConcept = entry.getKey();
			ArrayList<String> relatedConcepts = concepts.get(chosenConcept);
			ArrayList<String> cosineRelatedConcepts = reference.get(chosenConcept);
			TreeMap<String, String> pair = entry.getValue();
			
			Iterator pairIterator = pair.entrySet().iterator();
			ArrayList<String> temp = new ArrayList<String>();
			ArrayList<String> tempen = new ArrayList<String>();
			ArrayList<String> onlyCosine = new ArrayList<String>();
			ArrayList<String> onlyAR = new ArrayList<String>();
			while(pairIterator.hasNext()){
				Entry<String, String> pairEntry = (Entry<String, String>) pairIterator.next();
				String cosine = pairEntry.getKey();
				String ar = pairEntry.getValue();
				if (!ar.equals(chosenConcept)) {
					tempen.add(ar);
				}
				if (!cosine.equals(chosenConcept)) {
					temp.add(cosine);
				}
			}
			
			for (String c : cosineRelatedConcepts) {
				if (!temp.contains(c)) onlyCosine.add(c);
			}
			for (String a : relatedConcepts) {
				if (!tempen.contains(a)) onlyAR.add(a);
			}
			System.out.println();
			onlyInCosine.put(chosenConcept, onlyCosine);
			onlyInAR.put(chosenConcept, onlyAR);
			System.out.println("temep "+temp.size());
			System.out.println("temepen "+tempen.size());
			System.out.println("only ar "+onlyAR.size());
			if (chosenConcept.equals("control")) {
				for (String string : tempen) {
					System.out.println(string);
				}
			}
		}
	}
	
	public void printGroups() {
		Iterator iterator = inBoth.entrySet().iterator();
		while(iterator.hasNext()){
			Entry<String, TreeMap<String, String>> entry = (Entry<String, TreeMap<String, String>>) iterator.next();
			printFoundByBoth(entry); 
		}
		Iterator iterator2 = onlyInAR.entrySet().iterator();
		while(iterator2.hasNext()){
			Entry<String, ArrayList<String>> entry = (Entry<String, ArrayList<String>>) iterator2.next();
			printFounByAR(entry);	
		}
		
		Iterator iterator3 = onlyInCosine.entrySet().iterator();
		while(iterator3.hasNext()){
			Entry<String, ArrayList<String>> entry = (Entry<String, ArrayList<String>>) iterator3.next();
			printFoundByCosine(entry);
		}
	}

	private void printFounByAR(Entry<String, ArrayList<String>> entry) {
		String chosenConcept = entry.getKey();
		ArrayList<String> relatedConcepts = entry.getValue();		
		int nof0 = 0;
		int nof1 = 0;
		int nof2 = 0;
		System.out.println("--------- Found only by AR --------");
		String table = "";
		String chosen = chosenConcept.replaceAll(" ", "_");
		String url = "file:/C:/Documents and Settings/kvarv.WIN-NTNU-NO/Desktop/tables/"+chosen+"_onlyar.tex";
		table += "\\begin{table}[H]\n\\begin{center}\n\\small\n" +
				"\\textbf{Found only by the system for concept \""+chosenConcept+"\"} \\\\\n" +
		"\\begin{tabularx}{11cm}{|X|p{2.5cm}|}\n";
		table += "\\hline \n\\textbf{Related concept} & \\textbf{Mean score} \\\\ \n\\hline\n";

		for (String string : relatedConcepts) {
			int arMean = getArMean(chosenConcept, string);
			table += string + " & " + arMean + " \\\\\n";
			table += "\\hline\n";
			switch (arMean) {
			case 0:
				nof0++;
				break;
			case 1:
				nof1++;
				break;
			case 2:
				nof2++;
				break;
			default:
				break;
			}
		}
		table += "\\end{tabularx}\n" +
				"\\caption{Related concepts found only by the system for" +
				" the concept \""+chosenConcept+"\".}\n" +
				"\\label{tab:"+chosenConcept+"_onlyar}\n" +
				"\\end{center}\n" +
		"\\end{table}\n";
		
		System.out.println("	Number of related concepts	Not related	Related	Very related");
		System.out.println(chosenConcept+"	"+(nof0+nof1+nof2)+"	"+nof0+"	"+nof1+"	"+nof2+"\n");
		try {
			new TexWriter(new URL(url), table);
		} catch (MalformedURLException e) {
			e.printStackTrace();
		}
	}

	
	private void printFoundByCosine(Entry<String, ArrayList<String>> entry) {
		String chosenConcept = entry.getKey();
		ArrayList<String> relatedConcepts = entry.getValue();
		int nof0 = 0;
		int nof1 = 0;
		int nof2 = 0;

		System.out.println("--------- Found only by cosine similarity --------");
		String table = "";
		String chosen = chosenConcept.replaceAll(" ", "_");
		String url = "file:/C:/Documents and Settings/kvarv.WIN-NTNU-NO/Desktop/tables/"+chosen+"_onlycosine.tex";
		table += "\\begin{table}[H]\n\\begin{center}\n\\small\n" +
				"\\textbf{Found only by cosine similarity for concept \""+chosenConcept+"\"} \\\\\n" +
		"\\begin{tabularx}{11cm}{|X|p{2.5cm}|}\n";
		table += "\\hline \n\\textbf{Related concept} & \\textbf{Mean score} \\\\ \n\\hline\n";

		for (String string : relatedConcepts) {
			if (!string.equals(chosenConcept)) {
				int cosineMean = getCosineMean(chosenConcept, string);
				table += string + " & " + cosineMean + " \\\\\n";
				table += "\\hline\n";
				switch (cosineMean) {
				case 0:
					nof0++;
					break;
				case 1:
					nof1++;
					break;
				case 2:
					nof2++;
					break;
				default:
					break;
				}
			}
		}
		table += "\\end{tabularx}\n" +
				"\\caption{Related concepts found only by cosine similarity for" +
				" the concept \""+chosenConcept+"\".}\n" +
				"\\label{tab:"+chosenConcept+"_onlycosine}\n" +
				"\\end{center}\n" +
		"\\end{table}\n";
		System.out.println("	Number of related concepts	Not related	Related	Very related");
		System.out.println(chosenConcept+"	"+(nof0+nof1+nof2)+"	"+nof0+"	"+nof1+"	"+nof2+"\n");
		try {
			new TexWriter(new URL(url), table);
		} catch (MalformedURLException e) {
			e.printStackTrace();
		}
	}
	
	//"\\textbf{Found by both for concept \""+chosenConcept+"\"} \\\\\n" +

	private void printFoundByBoth(Entry<String, TreeMap<String, String>> entry) {
		String chosenConcept = entry.getKey();
		TreeMap<String, String> pair = entry.getValue();
		int nof0 = 0;
		int nof1 = 0;
		int nof2 = 0;
		
		Iterator pairIterator = pair.entrySet().iterator();
		System.out.println("--------- Found in both --------");
		String table = "";
		String chosen = chosenConcept.replaceAll(" ", "_");
		String url = "file:/C:/Documents and Settings/kvarv.WIN-NTNU-NO/Desktop/tables/"+chosen+"_both.tex";
		table += "\\begin{table}[H]\n\\begin{center}\n\\small\n" +
		"\\textbf{Found by both for concept \""+chosenConcept+"\"} \\\\\n" +
		"\\begin{tabularx}{11cm}{|X|p{2.5cm}|}\n";
		table += "\\hline \n\\textbf{Related concept} & \\textbf{Mean score} \\\\ \n\\hline\n";

		while(pairIterator.hasNext()){
			Entry<String, String> pairEntry = (Entry<String, String>) pairIterator.next();
			String cosine = pairEntry.getKey();
			String ar = pairEntry.getValue();
			if (!ar.equals(chosenConcept) || !cosine.equals(chosenConcept)) {
				if (ar.equals(cosine)) {
					int cosineMean = getCosineMean(chosenConcept, cosine);
					int arMean = getArMean(chosenConcept, ar);
					int mean;
					if(arMean != cosineMean) mean = Math.max(arMean, cosineMean);
					else mean = arMean;
					table += cosine + " & " + mean + " \\\\\n";
					table += "\\hline\n";
					switch (mean) {
					case 0:
						nof0++;
						break;
					case 1:
						nof1++;
						break;
					case 2:
						nof2++;
						break;
					default:
						break;
					}
				}/*
				else {
					int arMean = getArMean(chosenConcept, ar);
					int cosineMean = getCosineMean(chosenConcept, cosine);
					table += cosine + " & " + cosineMean + " \\\\\n";
					table += "Abstraction matched: " +ar + " & " + arMean + " \\\\\n";
				}*/
	
			}
		}
		table += "\\end{tabularx}\n" +
				"\\caption{Related concepts found by both the system and cosine similarity for" +
				" the concept "+chosenConcept+".}\n" +
				"\\label{tab:"+chosenConcept+"_byboth}\n" +
				"\\end{center}\n" +
				"\\end{table}\n";
		System.out.println("	Number of related concepts	Not related	Related	Very related");
		System.out.println(chosenConcept+"	"+(nof0+nof1+nof2)+"	"+nof0+"	"+nof1+"	"+nof2+"\n");
		try {
			new TexWriter(new URL(url), table);
		} catch (MalformedURLException e) {
			e.printStackTrace();
		}
	}
	
	/*
	public void splitIntoGroups(ArrayList<String> chosenConcepts, 
			TreeMap<String, ArrayList<String>> concepts, TreeMap<String, ArrayList<String>> reference) {
		for (String chosenConcept : chosenConcepts) {
			ArrayList<String> relatedConcepts = concepts.get(chosenConcept);
			ArrayList<String> cosineRelatedConcepts = reference.get(chosenConcept);
			
			ArrayList<String> onlyAR = new ArrayList<String>();
			ArrayList<String> both  = new ArrayList<String>();
			ArrayList<String> onlyCosine = new ArrayList<String>();
			for (String cosineRelatedConcept : cosineRelatedConcepts) {
				if (relatedConcepts.contains(cosineRelatedConcept) ||
						cosineRelatedConcept.equals(chosenConcept)) {
					both.add(cosineRelatedConcept);
				}
				else {
					onlyCosine.add(cosineRelatedConcept);
				}
			}
			for (String relatedConcept : relatedConcepts) {
				if (!cosineRelatedConcepts.contains(relatedConcept)) {
					onlyAR.add(relatedConcept);
				}
			}
			inBoth.put(chosenConcept, both);
			onlyInAR.put(chosenConcept, onlyAR);
			onlyInCosine.put(chosenConcept, onlyCosine);
			
			System.out.println("Only in cosine: " + chosenConcept.toUpperCase());
			for (String string : onlyCosine) {
				System.out.println("  "+string);
			}
			System.out.println("\nOnly in AR: " + chosenConcept.toUpperCase());
			for (String string : onlyAR) {
				System.out.println("  "+string);
			}
			System.out.println("\nIn both: " + chosenConcept.toUpperCase());
			for (String string : both) {
				System.out.println("  "+string);
			}
			System.out.println();
		}		
	}
	*/
	
	
	
	// Getters
	public TreeMap<String, TreeMap<String, String>> getInBoth() {
		return inBoth;
	}

	public TreeMap<String, ArrayList<String>> getNotByAR() {
		return onlyInAR;
	}

	public TreeMap<String, ArrayList<String>> getNotByCosine() {
		return onlyInCosine;
	}
	
	
	public static void main(String args[]) {
		MyConcepts myConcepts;
		CosineSimilarityResults cosineSimilarityResults;
		ManualOntologyConcepts manualOntologyConcepts;
		MyRelatedConcepts myRelatedConcepts;
		SemiOnotologyConcepts semiOnotologyConcepts;
		try {
			myConcepts = new MyConcepts(new URL("file:/C:/kvarv/Diplom/resultater/for real/" +
			"concepts - support=0.05, confidence=0.75, nofTerms=45.txt"));
			
			cosineSimilarityResults = new CosineSimilarityResults(new URL("file:/C:/kvarv/Diplom/resultater/for real/" +
						"cosine similarity.txt"));
			
			manualOntologyConcepts = new ManualOntologyConcepts(new URL("file:/C:/kvarv/Diplom/resultater/for real/" +
					"manual ontology concepts lemmatized.txt"));
			
			myRelatedConcepts = new MyRelatedConcepts(new URL("file:/C:/kvarv/Diplom/resultater/for real/" +
					"environments - support=0.05, confidence=0.75, nofTerms=45.txt"));
			
			semiOnotologyConcepts = new SemiOnotologyConcepts(new URL("file:/C:/kvarv/Diplom/resultater/for real/" +
					"PMBOK_semiauto.owl"));
			
			//Testing
			TestCases testCases = new TestCases();
			//testCases.checkConceptMatch(myConcepts.getConcepts(), semiOnotologyConcepts.getConcepts());
	
			ArrayList<String> chosenConcepts = new ArrayList<String>();
			chosenConcepts.add("cost");
			chosenConcepts.add("control");
			chosenConcepts.add("scope");
			chosenConcepts.add("risk");
			chosenConcepts.add("project plan");
			
			testCases.checkRelatedConceptMatch(chosenConcepts, myRelatedConcepts.getRelatedConceptsMap(), 
					cosineSimilarityResults.getRelatedConceptsMap());
			//testCases.splitIntoGroups(chosenConcepts, myRelatedConcepts.getRelatedConceptsMap(), 
				//	cosineSimilarityResults.getRelatedConceptsMap());
			TreeMap<String, TreeMap<String, String>> inBoth = testCases.getInBoth();
			testCases.splitToGroups(inBoth, myRelatedConcepts.getRelatedConceptsMap(), cosineSimilarityResults.getRelatedConceptsMap());
			testCases.printGroups();
			
		} catch (MalformedURLException e) {
			e.printStackTrace();
		}
	}


}
